<?php

namespace FiloBlu\Core\Plugin\Magento\Webapi\Model\Soap;

/**
 * Plugin needed to fix SOAP WSDL urls
 * https://your.magento.host/soap/all?wsdl_list=1
 */
class Server
{

    /**
     * @var \Magento\Framework\App\AreaList
     */
    protected $areaList;
    /**
     * @var \Magento\Framework\Config\ScopeInterface
     */
    protected $scopeInterface;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManagerInterface;
    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $requestInterface;

    /**
     * Server constructor.
     * @param \Magento\Framework\App\AreaList $areaList
     * @param \Magento\Framework\Config\ScopeInterface $scopeInterface
     * @param \Magento\Store\Model\StoreManagerInterface $storeManagerInterface
     * @param \Magento\Framework\App\RequestInterface $requestInterface
     */
    public function __construct(
        \Magento\Framework\App\AreaList $areaList,
        \Magento\Framework\Config\ScopeInterface $scopeInterface,
        \Magento\Store\Model\StoreManagerInterface $storeManagerInterface,
        \Magento\Framework\App\RequestInterface $requestInterface
    )
    {
        $this->areaList = $areaList;
        $this->scopeInterface = $scopeInterface;
        $this->storeManagerInterface = $storeManagerInterface;
        $this->requestInterface = $requestInterface;
    }

    /**
     * @param \Magento\Webapi\Model\Soap\Server $subject
     * @param $result
     * @return string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function afterGetEndpointUri(
        \Magento\Webapi\Model\Soap\Server $subject,
        $result
    )
    {

        $frontName = $this->areaList->getFrontName($this->scopeInterface->getCurrentScope());
        // Not in SOAP scope? Do nothing!
        if ($frontName !== 'soap') {
            return $result;
        }

        // Stripping store code from base url
        $currentStoreCode = $this->storeManagerInterface->getStore()->getCode();
        $baseUrl = $this->storeManagerInterface->getStore()->getBaseUrl();
        $strippedStoreCodeBaseUrl = str_replace("/{$currentStoreCode}", '', $baseUrl);

        // StoreCode of original getEndpointUri function
        $storeCode = $this->storeManagerInterface->getStore()->getCode() === \Magento\Store\Model\Store::ADMIN_CODE
            ? \Magento\Webapi\Controller\PathProcessor::ALL_STORE_CODE
            : $this->storeManagerInterface->getStore()->getCode();

        // Rebuild
        return "{$strippedStoreCodeBaseUrl}{$frontName}/{$storeCode}";
    }
}
